/*************************************************************************************************************
23th January 2008

This program (hosp_region.prg) is for a 3 equations model.


This program  estimates a multinomial logit model with 3 alternatives that depend on a dummy endogenous regressor (D).

A probit model for this dummy endogenous regressor (D) is specified.

One continous regressor of this probit model is only observed for those that have D=1. A linear model is specified for
this partially observed continous regressor.

The program allows for endogeneity through an individual level common factor unobservable variable and a region 
level unobservable variable, both are integrated out in the likelihood function.


The model is the one that I used for the paper with Angel Lopez on Subsidizing health insurance:

Multinomial Logit -> Type of health care use

Probit Model-> D-> Whether or not individual have private health insurance. The insurance premium is one of the regressors
                   of this Probit model but it is only observed for those with private health insurance

Linear Model-> A linear model for the insurance premium

Identification requires a variable in the Probit model that is excluded from the Linear Model. This would be equivalent
to the exclusion restriction required in the standard Heckman model. Notice, this variable would not need to be excluded
from the Multinomial Logit. Identification also requires the existence of another variable in the Linear Model or
in the Probit model that is excluded from the Multinomial Logit

Would the model be identified if we had one variable that affects the Probit but none of the other????


The integration is carried out using Halton sequences rather than random draws from the normal distribution.
I have found that the Halton sequences work considerably better than the random draws


**************************************************************************************************************/


@this one uses halton sequences@


new;

library optmum;
#include hessian.src;
#include grad2.g;
#include halton.prg;

output file = hosp_region_new.out reset;

/* Enter the number of simulation */
/* Introducir numero par */
r=500;


@----------------------- Files names ------------------------------------@
/* USER, enter here the names of the files to save the
the coefficients and the variance covariance matrix */


ficoef = "ch1_reg_new.fmt";
fimvc = "mvch1_reg_new.fmt";
namevar=setvars("newsample_region");
dataset="newsample_region";

@------------------------- Loading data ---------------------------------@
/*

Angel, aqui va la parte de leer los datos de la ESCA y la prima...
Pero en este fichero voy a utilizar datos completamente simulados...
asi que lo que hago en esta parte es fabricar estos datos simulados

d=1 significa seguro privado

p=prima
theta la vble no observable que figura en las dos ecuaciones
x otro regresor
*/



/* Names of dependent multinomial variables */
/*
y1=1 va al hosp publico, 0 caso contrario
y2=1 va al hosp privado, 0 caso contrario
y3=1 no va al hospital, 0 caso contrario
*/

namesy1={ y_hs1 };
namesy2={ y_hs2 };
namesy3={ y_hs3 };

/*Name of the vble that determines the unobservable at the region level */
/*This variable must take numerical values and be 1,2,3,....,NR, without any gaps */

name_region={ regio };

/* Name for the dependent variable of the probit and endogenous dummy in the multinomial */
namesd={ dssyp };

/*
 We should have variables y1, y2, y3, d(insurance status), and p(premium) and the rest of covariates (x)
namesxp= regressors for the probit. It must include the "premium" and constant term
namesy2= regressors for the specialist. It must include the insurance status "d" and constant term
namesy3= regressors fo the no public use. It must includes the insurance status "d" and constant term
*/

@Specify variables and initial conditions for the betas@


@Variables for probit. The premium must be the last one !!!!!@

namesxp={ constan, edat, edat2, edatpp, edatpp2, female, femov40, d_local, ed_prima, ed_secun, ed_uni,
 ed_h_prima, ed_h_secun, ed_h_uni, dacci, saluguay, salupuff, anycron, ncron, p1, inc4, inc5, inc6, inc7,  
inc8, inc9_12, lprinetp };

condxp={
    -1.1843067 
  -0.032311081 
     0.2350101 
    -0.2518218 
     0.3879914 
  -0.073107596 
    0.24707458 
     0.7210886 
    0.40447662 
    0.38383514 
     0.7801132 
   0.032288503 
    0.47835929 
     0.3217894 
   -0.23487002 
    0.17070747 
   -0.11704478 
 -0.0094017197 
 -0.0071850922 
   -0.72271981 
    0.54340967 
     1.0638023 
      1.479009 
     1.9710505 
      2.450771 
     3.3998295 
    -2.2159248
};

condxp=condxp';
condxp=1.225*condxp;

@Variables for the numerator of prob(y2) and prob(y3) respectively. y1 is taken as default@
namesz2={ constan, edat, edat2, edatpp, edatpp2, female, femov40, d_local, ed_prima, ed_secun, ed_uni,
 ed_h_prima, ed_h_secun, ed_h_uni, dacci, saluguay, salupuff, anycron, ncron, p1, inc4, inc5, inc6, inc7,
inc8, inc9_12, dssyp };

namesz3=namesz2;

condz2={
  -3.3556216     
  -1.1211054 
   1.0442076 
 -0.15648984 
 -0.12402252 
  0.55153547 
  0.15751238 
 0.080366446 
  0.50339783 
  0.44678699 
   1.0892312 
 -0.43388618 
0.0083012331 
-0.069843486 
-0.072110766 
  -0.2527237 
   -0.372597 
  0.21667983 
-0.086962609 
-0.048110608 
 -0.36449126 
  0.25010051 
  0.63450827 
  0.78822536 
  0.60109707 
  0.48817216 
  2.7121671 
 };
condz2=condz2';

condz3={
   2.6087943  
  0.11571508  
 -0.32142307  
  0.70951816  
  -0.5418712  
 -0.10274593  
  0.69716131  
  0.17387377  
-0.080062656  
 0.085540757  
 0.049088555  
 0.086584672  
 -0.04801788  
  0.23430029  
 -0.49861825  
  0.37023121  
 -0.65043352  
  -0.1451512  
 -0.23973352  
-0.044522292  
 0.044453666  
  0.27998463  
  0.20522572  
  0.41776089  
  0.48798858  
 0.048344816  
  0.48387058  
 };

condz3=condz3';

@Regressors for the premium equation@
namesxpre={ constan, edat, edat2, edatpp, edatpp2, female, femov40, d_local, ed_prima, ed_secun, ed_uni,
 ed_h_prima, ed_h_secun, ed_h_uni, dacci, saluguay, salupuff, anycron, ncron, p1, lcmedio };

condxpre={
     2.0286059
   -0.25326684
    0.13816004
     0.3319612
   -0.28718462
  -0.040350499
   0.010870922
   -0.26658982
  -0.017330206
   -0.10774776
   -0.31013048
   0.048516193
   -0.17110813
  -0.098904379
   0.048055519
  0.0048376928
   0.024250216
   0.090714879
  0.0024083575
   -0.14046822
   0.038594686
};
condxpre=condxpre';

@names for the loading factors of
-individual level unobservable for dependent variables y2, and y3= loady2 loady3 
-individual level unobservable for the premium equation = loadpre
-standard deviation of the error term of the premium=std_e_pr
-regional level unobservable variable for dependent variables y2, y3: regey2 regey3
-regional level unobservable variable for the premium equation =  regepre
@

names_wei={ loady2, loady3, loadpre, std_e_pr, regey2, regey3, regepre, regprob };

@Enter their initial conditions as specified in Names_wei. Do not introduce variaces@
cond_v={ 0.57, 0.39,  -0.33 ,  0.94, 0.05, -0.05, -0.05, 1 };



@---------- End of Loading data------------------------------@


@----------------------- Loading Variables -------------------------------@


open f1 = ^dataset for read;
call makevars(readr(f1,rowsf(f1)),0,namevar);
close(f1);


region=mergevar(name_region);

y1=mergevar(namesy1);
y2=mergevar(namesy2);
y3=mergevar(namesy3);
d=mergevar(namesd);


constan=ones(rows(y1),1);

xp=mergevar(namesxp);
z2=mergevar(namesz2);
z3=mergevar(namesz3);
xpre=mergevar(namesxpre);


@ Number of observations and parameters for each equation@
nobs=rows(y1);
kxp=cols(xp);     @number of covariates for the covariates of the probit@
kz2=cols(z2);     @number of covariates for the numerator of option y2==1@
kz3=cols(z3);     @number of covariates for the numerator of option y3==1@
kxpre=cols(xpre); @number of covariates in premium equation@


@---------------------------------------------------------------------@

@In the line below, we create a matrix of independent normal(0,1) with dimension Nx(half number of simulations)@
@This is the one for the unobservable variable that is in all the equations@
@In the line below, we create a matrix of independent normal(0,1) with dimension NxR using Halton sequences@
bigu=halton3(nobs,r,20);
bigu=cdfni(bigu);


@Now, we also create a matrix of independent normal(0,1) with the same dimension as before. This is to simulate
the premium for those that do not have insurance@
bigepre=halton5(nobs,r,20);
bigepre=cdfni(bigepre);

@Now, we also create a matrix of independent normal(0,1) with number of rows equals to number of regions.@
@This is to simulate the region level unobservable@
maxreg=maxc(region);
smallereg=halton7(maxreg,r,20);
smallereg=cdfni(smallereg);

print "Generating the Matrix of Region Level Unobservables";
@Constructing the matrix of region level unobservables@
bigereg=zeros(nobs,r);
j=0;
do until j==maxreg;
j=j+1;
@region j@
mat_reg_j=(ones(nobs,r).*(region.==j)).*smallereg[j,.];
bigereg=bigereg+mat_reg_j;
endo;
@bigereg will be the resulting matrix with the region level unobservables@
Print "Finished Generating the Matrix of Region Level Unobservables";




@-------------------Partition the data according to different events----------@


@Selecting matrices for event y1==1 and d==1@

xp_y1d1=selif(xp,y1 .and d);
z2_y1d1=selif(z2,y1 .and d);
z3_y1d1=selif(z3,y1 .and d);
u_y1d1=selif(bigu,y1 .and d);
xpre_y1d1=selif(xpre,y1 .and d);
r_y1d1=selif(bigereg, y1 .and d);

@Selecting matrices for event y1==1 and d==0@

xp_y1d0=selif(xp,y1 .and d.==0);
z2_y1d0=selif(z2,y1 .and d.==0);
z3_y1d0=selif(z3,y1 .and d.==0);
u_y1d0=selif(bigu,y1 .and d.==0);
e_y1d0=selif(bigepre,y1 .and d.==0); @this is to simulate the premium@
xpre_y1d0=selif(xpre,y1 .and d.==0);
r_y1d0=selif(bigereg, y1 .and d.==0);

@Selecting matrices for event y2==1 and d==1@

xp_y2d1=selif(xp,y2 .and d);
z2_y2d1=selif(z2,y2 .and d);
z3_y2d1=selif(z3,y2 .and d);
u_y2d1=selif(bigu,y2 .and d);
xpre_y2d1=selif(xpre,y2 .and d);
r_y2d1=selif(bigereg, y2 .and d);

@Selecting matrices for event y2==1 and d==0@

xp_y2d0=selif(xp,y2 .and d.==0);
z2_y2d0=selif(z2,y2 .and d.==0);
z3_y2d0=selif(z3,y2 .and d.==0);
u_y2d0=selif(bigu,y2 .and d.==0);
e_y2d0=selif(bigepre,y2 .and d.==0); @this is to simulate the premium@
xpre_y2d0=selif(xpre,y2 .and d.==0);
r_y2d0=selif(bigereg, y2 .and d.==0);

@Selecting matrices for event y3==1 and d==1@

xp_y3d1=selif(xp,y3 .and d);
z2_y3d1=selif(z2,y3 .and d);
z3_y3d1=selif(z3,y3 .and d);
u_y3d1=selif(bigu,y3 .and d);
xpre_y3d1=selif(xpre,y3 .and d);
r_y3d1=selif(bigereg,y3 .and d);


@Selecting matrices for event y3==1 and d==0@

xp_y3d0=selif(xp,y3 .and d.==0);
z2_y3d0=selif(z2,y3 .and d.==0);
z3_y3d0=selif(z3,y3 .and d.==0);
u_y3d0=selif(bigu,y3 .and d.==0);
e_y3d0=selif(bigepre,y3 .and d.==0); @this is to simulate the premium@
xpre_y3d0=selif(xpre,y3 .and d.==0);
r_y3d0=selif(bigereg,y3 .and d.==0);

@---------------------------------------------------------------------@
@Computing how many observations we have in each event@
obs_y1d1=rows(xp_y1d1);
obs_y1d0=rows(xp_y1d0);
obs_y2d1=rows(xp_y2d1);
obs_y2d0=rows(xp_y2d0);
obs_y3d1=rows(xp_y3d1);
obs_y3d0=rows(xp_y3d0);

if nobs ne obs_y1d1+obs_y1d0+obs_y2d1+obs_y2d0+obs_y3d1+obs_y3d0;
Print "There is something going wrong with the number of observations";
end;
endif;

@---------------------- Empieza el calculo de estadisticos ---------------@
"";
"We are trying to estimate a multinomial logit with endogenous probit";
"To account for endogeneity, we integrate out an unobservable variable that affects both the probit and the multinomial";
"The number of simulations is ";;r;
"";
"---------------------------Descriptive statistics-------------------------";
/* Esto lo podemos hacer cuando tengamos un dataset de gauss con las vbles
call dstat(dataset,0);
*/
@--------------------------Initial conditions--------------------------------@

@We stack in the same vector the initial conditions for the maximization of the likelihood@
theta0=condxp|condz2|condz3|condxpre|cond_v;

load ch1_reg;
theta0=ch1_reg|eye(1);

@Here we enter the names of the parameters that are maximized @

 _opparnm=namesxp
    |namesz2
    |namesz3
    |namesxpre
    |names_wei;

@---------------------- Globals for the Optimization ------------------------@
@These are things of the optimization routine, they would be different in stata@

@for optmum@
_opgtol=1e-4;
_opalgr=2;
__title="Multinomial with endogeneity";
__output=1;
add=1;


@----------------------------- Minimizacion -------------------------------@

"-------------------------------------------------------------------------";
"We are about to start the minimization";
"-------------------------------------------------------------------------";
_fmtnv={"*.*lg" 15 8};
__fmtcv={"-*.*s" 12 8};

print"============================================================";
print"        Initial Conditions for the Minimization             ";
print"============================================================";
print"Parameter       Value     ";
print"------------------------------------------------------------";
call printfmt(_opparnm~theta0,0~1);
print"------------------------------------------------------------";
print;
"";
"";


"The value of the objective function at the initial condition is ";
mle(theta0);


@here we start the optimization of the likelihood function which is called mle@
@the initial conditions are called theta0@

{thetaest,nf,g,retcode}=optprt(optmum(&mle,theta0));
save ^ficoef =thetaest;

@Esto es un chequeo de Gauss para saber que la optimizacion esta bien@
if retcode ne 0;
print " WARNING, RETCODE IS DIFFERENT FROM ZERO ";
end;
endif;



@-----We compute the VCV matrix of the parameter estimates----------@

print "Computing Jacobian of the VCV Matrix";
add=0;
jac=grad2(&mle,thetaest,mle(thetaest),0); /* Jacobian */
add=1;
print "Computing information matrix";
inf=jac'*jac;  /*Information matrix */
print "Inverting information matrix";
invinf=invpd(inf);
vcm=invinf;            /* outer product of gradients */
std=sqrt(diag(vcm));
"";
save ^fimvc =vcm;



@----------------------- Showing the results -----------------------@

thetashow=thetaest;
noms=namesxp
    |namesz2
    |namesz3
    |namesxpre
    |names_wei;
stdshow=std;

__fmtnv={"*.*lg" 15 8};
__fmtcv={"-*.*s" 12 8};

"Standard error computed using Outer product of the gradient";
print"============================================================";
print"                    Estimation Results                     ";
print"============================================================";
print"Parameter       Estimation     Std. Error         Z-Stat";
print"------------------------------------------------------------";

call printfmt(noms~thetashow~stdshow~(thetashow./stdshow),0~1~1~1);
print"------------------------------------------------------------";
print "--------------------------------------------------------";


end;

@*************************************************************************@
/*************************    PROCEDURES   *******************************/
@*************************************************************************@

/*************************************************************************
                          log likelihood
**************************************************************************/


proc mle(theta);

@Primero se definen las variables locales a la funcion de verosimilitud@
local loglike, betaxp, betaz2, betaz3, betaxpre, lo_2, lo_3, lo_pre, e_std;
local aaa, bbb, ccc, pd1, pd0, ddd2, ddd3, fff2, fff3, ggg2, ggg3, pre_simu, iii, hhh, jjj;
local py2, py2d1_n, py2d1, ly2d1;
local      py2d0_n, py2d0, ly2d0;
local py3, py3d0_n, py3d0, ly3d0;
local      py3d1_n, py3d1, ly3d1;
local py1, py1d1_n, py1d1, ly1d1;
local      py1d0_n, py1d0, ly1d0;
local      re_2, re_3, re_4, re_pre;
local      re_prob;

@Getting the parameters@

/*
The row vector beta includes all the parameters of the model. Here I split
them in the betas of the different parts of the models, as well as the load
factors of the individual unobservables (lo_2 and lo_3 lo_pre),
the standard deviation of the premium (e_std)
and the factors of the regional random effect (re_2 re_3 re_pre) 
re_prob is the standard deviation of the regional random effect in the probit equation */


betaxp=theta[1:kxp,1];
betaz2=theta[kxp+1:kxp+kz2,1];
betaz3=theta[kxp+kz2+1:kxp+kz2+kz3,1];
betaxpre=theta[kxp+kz2+kz3+1:kxp+kz2+kz3+kxpre,1];
lo_2=theta[kxp+kz2+kz3+kxpre+1,1];
lo_3=theta[kxp+kz2+kz3+kxpre+2,1];
lo_pre=theta[kxp+kz2+kz3+kxpre+3,1];
e_std=theta[kxp+kz2+kz3+kxpre+4,1];
re_2=theta[kxp+kz2+kz3+kxpre+5,1];
re_3=theta[kxp+kz2+kz3+kxpre+6,1];
re_pre=theta[kxp+kz2+kz3+kxpre+7,1];
re_prob=theta[kxp+kz2+kz3+kxpre+8,1];

@--------------------------------------------------@
@Getting the log likelihood of event y2==1 and d==1@
@--------------------------------------------------@

/*
aaa= the index of the probit, that is, the standard x beta. It is a row vector with dimension the
     number of people in this event
bbb= we create a matrix of ones with dimension the (number of people in this event)x(number of simulations)
     This matrix of ones is multiplied row by row by the probit index (aaa). So in each row we will have
     the index of the probit xbeta of the individual of that row repeated across all the columns

ccc= we sum the index of the probit with the INDIVIDUAL unobservable variable (u_y2d1) AND REGIONAL UNOBSERVABLE

pd1= we apply the function FI (cumulative distribution function of the normal) to obtain the probability of
     d=1 as a function of both the xbeta and the unobservable variable!!!!! Notice that the dimension of this matrix is
(number of people in this event)x(number of simulations), because we have a different probability for each individual
and simulation

ddd2 and ddd3 = Notice that to compute p(y2=1) of the multinomial logit we need both beta2*Z and beta3*Z
               (beta1 is 0 because we take it as the default category). This is computed in ddd2 and ddd3

fff2 and fff3= we use the same trick as before with the matrix of ones. So, in each row we have the index
of the multinomial logit repeated in all the columns

ggg2 and ggg3 = we add the unobservable variables (REGIONAL and INDIVIDUAL) to the index of the multinomial logit.
                Notice that we have to multiply by the corresponding weighting factor


iii= is the index (x*beta) of the premium equation, including the INDIVIDUAL AND REGIONAL unobservable variable.
 That is why we also use the trick of the matrix of ones

jjj is the density of the normal evaluated at the premium. Because we have many means for each individual (due to
the simulation of the unobservable variable), we have to use the trick of the matrix of ones.
Notice that the last element of the matrix xp has to be the premium (I wrote it like that in the instructions)

py2 is the probabability of y2==1 with the multinomial logit. Notice that both the index and the unobservable
enter through ggg2 and ggg3. Notice that the dimension of this matrix is
(number of people in this event)x(number of simulations), because we have a different probability for each individual
and simulation

py2d1_n= we multimply each element (i,j) of the matrix py2 by the corresponding element (i,j) of the matrix pd1
So The resulting matrix is dimension (number of people in this event)x(number of simulations).
Then we multiplyit by the density of the normal to consider the probability of the premium

py2d1 = This is a row vector, with the number of rows equals the number of people in this event.
 Each row is the mean of the row py2_d1. Here we are integrating out the unobservable
variable

ly2d1= just takes the logaritmo neperiano of py2d1


*/

@Computing the part of the likelihood that refers to the probit@

aaa=xp_y2d1*betaxp;
bbb=ones(obs_y2d1,r).*aaa;
ccc=bbb+u_y2d1+r_y2d1*re_prob;
pd1=cdfn(ccc);

@Computing the part that refers to the multinomial logit@

ddd2=z2_y2d1*betaz2;
ddd3=z3_y2d1*betaz3;

fff2=ones(obs_y2d1,r).*ddd2;
fff3=ones(obs_y2d1,r).*ddd3;

ggg2=fff2+u_y2d1*lo_2+r_y2d1*re_2;
ggg3=fff3+u_y2d1*lo_3+r_y2d1*re_3;

py2=exp(ggg2)./(1+exp(ggg2)+exp(ggg3));

@Computing the part that refers to the premium@

hhh=xpre_y2d1*betaxpre;
iii=ones(obs_y2d1,r).*hhh+u_y2d1*lo_pre+r_y2d1*re_pre; @average of the premium equation. It includes the unobservable vble@
/*
jjj is the density of the normal evaluated at the premium. Because we have many means for each individual (due to
the simulation of the unobservable variable), we have to use the trick of the matrix of ones.
Notice that the last element of the matrix xp has to be the premium (I wrote it like that in the instructions)
*/
jjj=dens_nor(xp_y2d1[.,kxp].*ones(obs_y2d1,r),iii,e_std);

@mutliplying element by element the probability and densities of all the endogenous variables, conditional
on the unobservable variable @

py2d1_n=pd1.*py2;
py2d1_n=py2d1_n.*jjj;

@Averaging out the unobserved variable@

py2d1=meanc(py2d1_n');
ly2d1=ln(py2d1);

clear aaa, bbb, ccc, pd1, ddd2, ddd3, fff2, fff3, ggg2, ggg3, py2, py2d1_n, hhh, iii, jjj;

@-------------------------------------------------   @
@Getting the log likelihood of event y2==1 and d==0  @
@-------------------------------------------------   @
@We do not observe the premium for these individuals@

@this is the simulated premium. Notice that it has number of columns equal to the number of simulations
because we are using the unobserved variables here@

pre_simu=(xpre_y2d0*betaxpre).*ones(obs_y2d0,r)+u_y2d0*lo_pre+r_y2d0*re_pre+e_y2d0*e_std;

@Here we are doing the part of the probit@
@Here we get the regressors of the probit but we exclude the premium@
aaa=xp_y2d0[.,1:kxp-1]*betaxp[1:kxp-1,1];

@here we add the part of the premium to the other regressors of the probit@
bbb=ones(obs_y2d0,r).*aaa+betaxp[kxp,1]*pre_simu;
ccc=bbb+u_y2d0+r_y2d0*re_prob;
pd0=1-cdfn(ccc);

@Here we start the part of the multinomial logit@

ddd2=z2_y2d0*betaz2;
ddd3=z3_y2d0*betaz3;

fff2=ones(obs_y2d0,r).*ddd2;
fff3=ones(obs_y2d0,r).*ddd3;

ggg2=fff2+u_y2d0*lo_2+r_y2d0*re_2;
ggg3=fff3+u_y2d0*lo_3+r_y2d0*re_3;


py2=exp(ggg2)./(1+exp(ggg2)+exp(ggg3));

@Here we are multiplying the probabilities of NO insurance and y2. Notice that the pd0 does not only vary
because of the unobserved variable common to all the equations, but also due to the premium. So, we will be
implicitly integrating out the premium because we do not know it@

py2d0_n=pd0.*py2;
py2d0=meanc(py2d0_n');
ly2d0=ln(py2d0);


clear aaa, bbb, ccc, pd0, ddd2, ddd2, fff2, fff3, ggg2, ggg3, py2, py2d0_n, pre_simu;

@-------------------------------------------------   @
@Getting the log likelihood of event y3==1 and d==0  @
@-------------------------------------------------   @
@We do not observe the premium for these individuals@

@this is the simulated premium. Notice that it has number of columns equal to the number of simulations
because we are using the unobserved variables here@

pre_simu=(xpre_y3d0*betaxpre).*ones(obs_y3d0,r)+u_y3d0*lo_pre+r_y3d0*re_pre+e_y3d0*e_std;

@Here we are doing the part of the probit@
@Here we get the regressors of the probit but we exclude the premium@
aaa=xp_y3d0[.,1:kxp-1]*betaxp[1:kxp-1,1];

@here we add the part of the premium to the other regressors of the probit@
bbb=ones(obs_y3d0,r).*aaa+betaxp[kxp,1]*pre_simu;
ccc=bbb+u_y3d0+r_y3d0*re_prob;
pd0=1-cdfn(ccc);

@Here, we start the part on the multinomial logit@

ddd2=z2_y3d0*betaz2;
ddd3=z3_y3d0*betaz3;

fff2=ones(obs_y3d0,r).*ddd2;
fff3=ones(obs_y3d0,r).*ddd3;

ggg2=fff2+u_y3d0*lo_2+r_y3d0*re_2;
ggg3=fff3+u_y3d0*lo_3+r_y3d0*re_3;


py3=exp(ggg3)./(1+exp(ggg2)+exp(ggg3));

py3d0_n=pd0.*py3;
py3d0=meanc(py3d0_n');
ly3d0=ln(py3d0);

clear aaa, bbb, ccc, pd0, ddd2, ddd3, fff2, fff3, ggg2, ggg3, py3, py3d0_n, pre_simu;

@-------------------------------------------------   @
@Getting the log likelihood of event y3==1 and d==1  @
@-------------------------------------------------   @

@Part dedicated to the insurance probit@

aaa=xp_y3d1*betaxp;
bbb=ones(obs_y3d1,r).*aaa;
ccc=bbb+u_y3d1+r_y3d1*re_prob;
pd1=cdfn(ccc);

@Part dedicated to the multinomial logit@

ddd2=z2_y3d1*betaz2;
ddd3=z3_y3d1*betaz3;

fff2=ones(obs_y3d1,r).*ddd2;
fff3=ones(obs_y3d1,r).*ddd3;

ggg2=fff2+u_y3d1*lo_2+r_y3d1*re_2;
ggg3=fff3+u_y3d1*lo_3+r_y3d1*re_3;

py3=exp(ggg3)./(1+exp(ggg2)+exp(ggg3));

@Part dedicated to the premium@

hhh=xpre_y3d1*betaxpre;
iii=ones(obs_y3d1,r).*hhh+u_y3d1*lo_pre+r_y3d1*re_pre;

@density of the normal evaluated at the premium. Notice the mean includes the unob heterogeneity@
jjj=dens_nor(xp_y3d1[.,kxp].*ones(obs_y3d1,r),iii,e_std);

@mutliplying element by element the probability and densities of all the endogenous variables, conditional
on the unobservable variable @

py3d1_n=pd1.*py3;
py3d1_n=py3d1_n.*jjj;

@Averaging out the unobserved variable@

py3d1=meanc(py3d1_n');
ly3d1=ln(py3d1);

clear aaa, bbb, ccc, pd1, ddd2, ddd3, fff2, fff3, ggg2, ggg3, py3, py3d1_n;
clear hhh,iii,jjj;

@-------------------------------------------------   @
@Getting the log likelihood of event y1==1 and d==1  @
@-------------------------------------------------   @

@Part dedicated to the probit of insurance@

aaa=xp_y1d1*betaxp;
bbb=ones(obs_y1d1,r).*aaa;
ccc=bbb+u_y1d1+r_y1d1*re_prob;
pd1=cdfn(ccc);

@Part dedicated to the multinomial logit@

ddd2=z2_y1d1*betaz2;
ddd3=z3_y1d1*betaz3;

fff2=ones(obs_y1d1,r).*ddd2;
fff3=ones(obs_y1d1,r).*ddd3;

ggg2=fff2+u_y1d1*lo_2+r_y1d1*re_2;
ggg3=fff3+u_y1d1*lo_3+r_y1d1*re_3;

@This is because y1 is the default category@
py1=1-exp(ggg2)./(1+exp(ggg2)+exp(ggg3))-exp(ggg3)./(1+exp(ggg2)+exp(ggg3));

@Part dedicated to the premium@
hhh=xpre_y1d1*betaxpre;
iii=ones(obs_y1d1,r).*hhh+u_y1d1*lo_pre+r_y1d1*re_pre;
jjj=dens_nor(xp_y1d1[.,kxp].*ones(obs_y1d1,r),iii,e_std);

@multiplying the probabilities/densities of all the endogenous variables, conditional on the unobserved vble @
py1d1_n=pd1.*py1;
py1d1_n=py1d1_n.*jjj;

@averaging out the unobserved variable@
py1d1=meanc(py1d1_n');

ly1d1=ln(py1d1);


clear aaa, bbb, ccc, pd1, ddd2, ddd3, fff2, fff3, ggg2, ggg3, py1, py1d1_n, hhh, iii, jjj;

@-------------------------------------------------   @
@Getting the log likelihood of event y1==1 and d==0  @
@-------------------------------------------------   @
@We do not observe the premium for these individuals@

@this is the simulated premium. Notice that it has number of columns equal to the number of simulations
because we are using the unobserved variables here@

pre_simu=(xpre_y1d0*betaxpre).*ones(obs_y1d0,r)+u_y1d0*lo_pre+r_y1d0*re_pre+e_y1d0*e_std;

@Here we are doing the part of the probit@
@Here we get the regressors of the probit but we exclude the premium@
aaa=xp_y1d0[.,1:kxp-1]*betaxp[1:kxp-1,1];

@here we add the part of the premium to the other regressors of the probit@
bbb=ones(obs_y1d0,r).*aaa+betaxp[kxp,1]*pre_simu;
ccc=bbb+u_y1d0+r_y1d0*re_prob;
pd0=1-cdfn(ccc);

@here we start with the multinomial part@

ddd2=z2_y1d0*betaz2;
ddd3=z3_y1d0*betaz3;

fff2=ones(obs_y1d0,r).*ddd2;
fff3=ones(obs_y1d0,r).*ddd3;

ggg2=fff2+u_y1d0*lo_2+r_y1d0*re_2;
ggg3=fff3+u_y1d0*lo_3+r_y1d0*re_3;

@This is because y1 is the default category@
py1=1-exp(ggg2)./(1+exp(ggg2)+exp(ggg3))-exp(ggg3)./(1+exp(ggg2)+exp(ggg3));

py1d0_n=pd0.*py1;
py1d0=meanc(py1d0_n');
ly1d0=ln(py1d0);

clear aaa, bbb, ccc, pd0, ddd2, ddd3, fff2, fff3, ggg2, ggg3, py1, py1d0_n, pre_simu;

loglike=ly2d1|ly2d0|ly3d1|ly3d0|ly1d1|ly1d0;


 if add==1;

        retp(-sumc(loglike));
    elseif add==0;

       retp(loglike);

    endif;

endp;



@ This procedure returns the density function of the Normal (NU,desvstan) evaluated at x@
proc dens_nor(x,nu,desvstan);
    local a,b,c;
    a=((x-nu)/desvstan)^2;
    b=exp(-0.5*a);
    c=1/(desvstan*sqrt(2*pi))*b;
    retp(c);
endp;
